%  Le bras de robot le plus simple (pierre.rouchon@ensmp.fr, octobre 2007)
%  planification et suivi de trajectoire, robustesse, terme intgral,
%  anti- emballement
%  modle de simulation 
%     dth/dt = w
%     dw/dt = uv*(p1+eta1)-(p2+eta2)*sin(th) + eta + sig*randn
%     duv/dt= (u-uv)/epsilon
%  modle de contrle
%     dth/dt = w
%     dw/dt = u*p1-p2*sin(th) 
%  avec p1, p2 paramtres et eta1 et eta2 les erreurs
clear all, close all;
% paramtres du modle  et borne  sur u
T_period=1; 
p1=1; p2=(2*pi/T_period)^2; u_max=1.5*p2/p1; u_min=-u_max;
% erreur paramtrique, bruit, dyn moteur 
eta1=0.2*p1;  eta2=-0.3*p2;  eta=0.2*p2/p1;  sig=0.3*p2/p1 ; epsilon=0.05*T_period; 
% Gain contrleur
 xi=1.5;   w0=sqrt(p2);  Tint=3/w0;
% paramtres simu
dt=min(epsilon,T_period/100) ;ti=-0.2*T_period; tf=2*T_period; nt=floor((tf-ti)/dt);
% condition initiale modle simu et intgral controleur 
t=ti; th=0; w=0; uv=0;   Int=0; 
% initiatilsation buffer stockage trajectoires
t_b=t;th_b=th;w_b=w;uv_b=0;u_b=0;thr_b=th;wr_b=w;ur_b=0; 
% boucle de simu 
for iter=1:nt
        % trajectoire de reference sur [0, T] 
        T=0.7;   thf=pi;
        if(t<0)
            thr=0;Dthr=0;D2thr=0;
        elseif( t < T)
            s=t/T; thr=thf* (3*(s^2)-2*(s^3));  Dthr= thf*6*s*(1-s)/T;  D2thr=thf*(6-12*s)/(T^2);
        else 
            thr=thf;Dthr=0;D2thr=0;
        end
        wr=Dthr;  ur=(D2thr+p2*sin(thr))/p1;
        % linarisation exacte  de la dynamique de l'erreur de suivi 
        % rajout intgrateur pour l'erreur statique et prise en compte
        % saturation  sur u et aussi limitation sur Int
        thc=thr+Int; 
        Int=Int+dt*(thr-th)/Tint; 
        Int=max(-pi/2,min(+pi/2,Int)); % limitation correction refrence   +- pi/2      
        u= (p2*sin(th) + D2thr -2*xi*w0*(w-wr) - w0^2*sin(th-thc))/p1;
        u=max(u_min,min(u_max,u));
        % u=(p2*sin(thr)+D2thr)/p1;
        % integration via Euler explicite
        Dth=w;
        Dw= uv*(p1+eta1)-(p2+eta2)*sin(th) + eta + sig*randn ;
        Duv= (u-uv)/epsilon;
        th=th+dt*Dth; w=w+dt*Dw; uv=uv+dt*Duv; t=t+dt;
       % sauvegarde  graphique
        t_b=[t_b,t];  th_b=[th_b, th];  w_b=[w_b, w]; thr_b=[thr_b, thr];
        wr_b=[wr_b, wr]; uv_b=[uv_b, uv]; u_b=[u_b, u]; ur_b=[ur_b, ur];   
end

figure(1),clf,
subplot(311)
plot(t_b,th_b,t_b,thr_b,'r','LineWidth',2.5);
legend('\theta','\theta_r');
xlabel('temps (s)')
ylabel('rad')

subplot(312)
plot(t_b,w_b,t_b,wr_b,'r','LineWidth',2.5);
legend('\omega','\omega_r');
xlabel('temps (s)')
ylabel('Rad/s')
 
subplot(313)
plot(t_b,uv_b,t_b,u_b,t_b,ur_b,t_b,u_max*ones(1,length(t_b)),'--',t_b,u_min*ones(1,length(t_b)),'--','LineWidth',2.5);
legend('u_v_r_a_i','u','u_r');
xlabel('temps (s)')
ylabel('N/m')


return










   



